@@ -15,10 +15,11 @@ from starlette.types import ASGIApp, Receive, Scope, Send
 from kupala.config import Config
 from kupala.console.application import ConsoleApplication
 from kupala.container import Container, Resolver
-from kupala.contracts import ContextProcessor, Invoker
-from kupala.errors import ServerErrorMiddleware
-from kupala.exceptions import ErrorHandler, ExceptionMiddleware
+from kupala.contracts import Invoker
 from kupala.middleware import Middleware, MiddlewareStack
+from kupala.middleware.errors import ServerErrorMiddleware
+from kupala.middleware.exception import ErrorHandler, ExceptionMiddleware
+from kupala.middleware.template_context import TemplateContextMiddleware
 from kupala.requests import Request
 from kupala.responses import Response
 from kupala.routing import Router, Routes
@@ -38,7 +39,6 @@ class Kupala:
         template_dirs: list[str] = None,
         providers: t.Iterable[t.Union['Provider', t.Type['Provider']]] = None,
         routes: t.Union[Routes, list[BaseRoute]] = None,
-        context_processors: list[ContextProcessor] = None,
         commands: list[click.Command] = None,
         error_handlers: dict[t.Union[t.Type[Exception], int], t.Optional[ErrorHandler]] = None,
         middleware: t.Union[list[Middleware], MiddlewareStack] = None,
@@ -51,7 +51,6 @@ class Kupala:
         self.debug = debug
         self.middleware = MiddlewareStack(list(middleware or []))
         self.providers = providers or []
-        self.context_processors: t.List[ContextProcessor] = context_processors or []
         self.error_handlers = error_handlers or {}
         self.services = Container()
         self.template_dirs = template_dirs or []
@@ -99,6 +98,10 @@ class Kupala:
 
         request_container = Container(parent=self.services)
         scope["app"] = self
+        scope['state'] = {
+            'app': self,
+        }
+
         scope['resolver'] = request_container
         scope["router"] = self._router
 
@@ -119,7 +122,7 @@ class Kupala:
 
     def cli(self) -> int:
         set_current_application(self)
-        app = ConsoleApplication(self.services, self.commands)
+        app = ConsoleApplication(self, self.commands)
 
         with self.services.change_context({}):
             return app.run()
@@ -140,6 +143,7 @@ class Kupala:
         app: ASGIApp = Router(routes=self.routes)
         self._router = t.cast(Router, app)
         self.middleware.use(ExceptionMiddleware, handlers=self.error_handlers)
+        self.middleware.use(TemplateContextMiddleware)
         self.middleware.top(ServerErrorMiddleware, debug=self.debug, handler=self.exception_handler)
         for mw in reversed(self.middleware):
             app = mw.wrap(app)
