@@ -156,6 +156,14 @@ class FilesData:
         return t.cast(list[UploadFile], self._files.getlist(key))
 
 
+class Cookies(dict):
+    pass
+
+
+class Headers(requests.Headers):
+    pass
+
+
 class Request(requests.Request):
     def __new__(cls, scope: Scope, receive: Receive = empty_receive, send: Send = empty_send) -> Request:
         if 'request' not in scope:
@@ -236,6 +244,14 @@ class Request(requests.Request):
     def language(self) -> str:
         return self.locale.language
 
+    @property
+    def cookies(self) -> Cookies:
+        return Cookies(**super().cookies)
+
+    @property
+    def headers(self) -> Headers:
+        return Headers(super().headers)
+
     @property
     def old_input(self) -> OldFormInput:
         """Get previous form input. This value does not include uploaded files."""
