@@ -3,23 +3,13 @@ import typing as t
 from kupala import responses
 from kupala.container import ServiceNotFoundError
 from kupala.contracts import TemplateRenderer
-from kupala.csrf import get_csrf_input, get_csrf_token
-from kupala.messages import FlashBag, flash
 from kupala.requests import Request
 
 
 def default_app_context(request: Request) -> t.Mapping:
     return {
-        'debug': request.app.debug,
-        'url': request.url_for,
-        'config': request.app.config,
-        'form_errors': request.form_errors,
-        'old_input': request.old_input,
-        'csrf_token': get_csrf_token(request),
-        'csrf_input': get_csrf_input(request),
         'auth': request.auth,
         'user': request.user,
-        'messages': flash(request) if 'flash_messages' in request.scope else FlashBag(),
     }
 
 
@@ -38,13 +28,21 @@ class TemplateResponse(responses.Response):
         headers: dict = None,
     ) -> None:
         try:
-            context = context or {}
-            full_context = {'request': request}
+            full_context = {
+                'request': request,
+                'config': request.app.config,
+                'debug': request.app.debug,
+                'url': request.url_for,
+                'form_errors': request.form_errors,
+                'old_input': request.old_input,
+            }
+            try:
+                full_context.update(request.state.template_context)
+            except AttributeError:
+                pass
+
+            full_context.update(context or {})
             renderer: TemplateRenderer = request.app.resolve(TemplateRenderer)
-            context_processors = request.app.context_processors
-            for context_processor in context_processors:
-                full_context.update(context_processor(request))
-            full_context = {**full_context, **context}
             content = renderer.render(template_name, full_context)
             super().__init__(content, status_code, headers, media_type)
         except ServiceNotFoundError as ex:
